@@ -82,13 +82,13 @@ himasis <- function(Y, M, X, COV, glm.family, modelstatement,
     modelstatement <- modelstatement
   } else {
     COV <- data.frame(COV)
-    COV <- data.frame(model.matrix(~., COV))[, -1]
+    COV <- data.frame(model.matrix(~., COV))[, -1, drop = FALSE]
     conf.names <- colnames(COV)
     if (verbose) message("    Adjusting for covariate(s): ", paste0(conf.names, collapse = ", "))
-    datarun <- data.frame(Y = Y, Mone = NA, X = X, COV = COV)
+    datarun <- data.frame(Y = Y, Mone = NA, X = X, COV)
     modelstatement <- eval(parse(text = (paste0(
       modelstatement, "+",
-      paste0(paste0("COV.", conf.names), collapse = "+")
+      paste0(colnames(COV), collapse = "+")
     ))))
   }
 
@@ -137,7 +137,7 @@ process_var <- function(var, scale) {
 
 
 
-# Internal function: LDPE
+# Internal function: LDPE (optimized)
 # the code of Liu han's JRSSB paper for high-dimensional Cox model
 # ID: the index of interested parameter
 # X: the covariates matrix with n by p
@@ -145,186 +145,148 @@ process_var <- function(var, scale) {
 # status: the censoring indicator I(T <= C)
 
 LDPE_func <- function(ID, X, OT, status) {
+  set.seed(1029)
+
+  # Dimensions
   coi <- ID
-  x <- X
-  d <- dim(x)[2]
-  n <- dim(x)[1]
+  d <- ncol(X)
+  n <- nrow(X)
 
-  ## Set of tuning parameters
-  PF <- matrix(1, 1, d)
+  # Initialize penalty factor
+  PF <- rep(1, d)
   PF[ID] <- 1
-  fit <- glmnet(x, survival::Surv(OT, status), family = "cox", alpha = 1, standardize = FALSE, penalty.factor = PF)
-  cv.fit <- cv.glmnet(x, survival::Surv(OT, status), family = "cox", alpha = 1, standardize = FALSE, penalty.factor = PF)
-  betas <- coef(fit, s = cv.fit$lambda.min)[1:d] # the semi-penalized initial estimator  # initial estimator
 
-  stime <- sort(OT) # Sorted survival/censored times
-  otime <- order(OT) # Order of time
+  # Semi-penalized initial estimator
+  fit <- glmnet(X, survival::Surv(OT, status), family = "cox", alpha = 1, standardize = FALSE, penalty.factor = PF)
+  cv.fit <- cv.glmnet(X, survival::Surv(OT, status), family = "cox", alpha = 1, standardize = FALSE, penalty.factor = PF)
+  betas <- as.vector(coef(fit, s = cv.fit$lambda.min))
 
-  Vs <- matrix(rep(0, d * d), nrow = d)
-  Hs <- Vs # Hessian
-  ind <- 0
+  # Precompute sorted times and order
+  stime <- sort(OT)
+  otime <- order(OT)
 
-  la <- rep(0, n) # Gradient w.r.t parameter of interest
-  lb <- matrix(rep(0, (d - 1) * n), nrow = n) # Gradient w.r.t nuisance parameter (theta)
-  i <- 1
+  # Preallocate storage
+  la <- numeric(n)
+  lb <- matrix(0, n, d - 1)
+  Hs <- matrix(0, d, d)
 
-  while (i <= n) {
+  # Iterate through observations
+  for (i in seq_len(n)) {
     if (status[otime[i]] == 1) {
       ind <- which(OT >= stime[i])
-      S0 <- 0
-      S1 <- rep(0, d)
-      S2 <- matrix(rep(0, d * d), nrow = d)
+      X_ind <- X[ind, , drop = FALSE] 
+      tmp_exp <- as.vector(exp(X_ind %*% betas)) 
 
-      if (length(ind) > 0) {
-        for (j in seq_along(ind))
-        {
-          tmp <- exp(x[ind[j], ] %*% betas)
-          S0 <- S0 + tmp
+      S0 <- sum(tmp_exp)
+      S1 <- colSums(X_ind * tmp_exp)  
+      S2 <- t(X_ind) %*% (X_ind * tmp_exp)
 
-          S1 <- S1 + tmp %*% t(x[ind[j], ])
+      la[i] <- -(X[otime[i], coi] - S1[coi] / S0)
 
-          tmp <- apply(tmp, 1, as.numeric)
-          S2 <- S2 + tmp * x[ind[j], ] %*% t(x[ind[j], ])
-        }
-      }
-      S0 <- apply(S0, 1, as.numeric)
-
-      la[i] <- -(x[otime[i], coi] - S1[coi] / S0)
       if (coi == 1) {
-        lb[i, ] <- -(x[otime[i], c((coi + 1):d)] - S1[c((coi + 1):d)] / S0)
+        lb[i, ] <- -(X[otime[i], -1] - S1[-1] / S0)
       } else if (coi == d) {
-        lb[i, ] <- -(x[otime[i], c(1:(coi - 1))] - S1[c(1:(coi - 1))] / S0)
+        lb[i, ] <- -(X[otime[i], -d] - S1[-d] / S0)
       } else {
-        lb[i, ] <- -(x[otime[i], c(1:(coi - 1), (coi + 1):d)] - S1[c(1:(coi - 1), (coi + 1):d)] / S0)
+        lb[i, ] <- -(X[otime[i], -c(coi)] - S1[-c(coi)] / S0)
       }
-      V <- S0 * S2 - t(S1) %*% (S1)
-      Hs <- Hs + V / (n * S0^2)
+
+      Hs <- Hs + (S0 * S2 - tcrossprod(S1)) / (n * S0^2)
     }
-    i <- i + 1
   }
 
-  fit <- glmnet(lb, la, alpha = 1, standardize = FALSE, intercept = FALSE, lambda = sqrt(log(d) / n))
-  what <- as.numeric(coef(fit)[2:d])
+  # De-biased Lasso step
+  fit_res <- glmnet(lb, la, alpha = 1, standardize = FALSE, intercept = FALSE, lambda = sqrt(log(d) / n))
+  what <- as.vector(coef(fit_res)[-1])
 
+  # Final estimate and variance
   if (coi == 1) {
-    S <- betas[coi] - (mean(la) - t(what) %*% (colMeans(lb))) / (Hs[coi, coi] - t(what) %*% Hs[c((coi + 1):d), coi])
-    var <- Hs[coi, coi] - t(what) %*% Hs[c((coi + 1):d), coi]
+    S <- betas[coi] - (mean(la) - crossprod(what, colMeans(lb))) / (Hs[coi, coi] - crossprod(what, Hs[-1, coi]))
+    var <- Hs[coi, coi] - crossprod(what, Hs[-1, coi])
   } else if (coi == d) {
-    S <- betas[coi] - (mean(la) - t(what) %*% (colMeans(lb))) / (Hs[coi, coi] - t(what) %*% Hs[c(1:(coi - 1)), coi])
-    var <- Hs[coi, coi] - t(what) %*% Hs[c(1:(coi - 1)), coi]
+    S <- betas[coi] - (mean(la) - crossprod(what, colMeans(lb))) / (Hs[coi, coi] - crossprod(what, Hs[-d, coi]))
+    var <- Hs[coi, coi] - crossprod(what, Hs[-d, coi])
   } else {
-    S <- betas[coi] - (mean(la) - t(what) %*% (colMeans(lb))) / (Hs[coi, coi] - t(what) %*% Hs[c(1:(coi - 1), (coi + 1):d), coi])
-    var <- Hs[coi, coi] - t(what) %*% Hs[c(1:(coi - 1), (coi + 1):d), coi]
+    S <- betas[coi] - (mean(la) - crossprod(what, colMeans(lb))) / (Hs[coi, coi] - crossprod(what, Hs[-c(coi), coi]))
+    var <- Hs[coi, coi] - crossprod(what, Hs[-c(coi), coi])
   }
 
   beta_est <- S
   beta_SE <- sqrt(1 / (n * var))
 
   result <- c(beta_est, beta_SE)
-
   return(result)
 }
 
 
-
 # Internal function: null_estimation
 # A function to estimate the proportions of the three component nulls
-# This is from HDMT package version < 1.0.4
+# This is from HDMT package version < 1.0.4 (optimized here)
 
 null_estimation <- function(input_pvalues, lambda = 0.5) {
-  ## input_pvalues is a matrix with 2 columns of p-values, the first column is p-value for exposure-mediator association, the second column is p-value for mediator-outcome association adjusted for exposure
-  ## lambda is the threshold for pi_{00} estimation, default 0.5
-  # check input
-  if (is.null(ncol(input_pvalues))) {
-    stop("input_pvalues should be a matrix or data frame")
+  ## Validate input
+  if (is.null(ncol(input_pvalues)) || ncol(input_pvalues) != 2) {
+    stop("`input_pvalues` must be a matrix or data frame with exactly 2 columns.")
   }
-  if (ncol(input_pvalues) != 2) {
-    stop("inpute_pvalues should have 2 column")
+  input_pvalues <- as.matrix(input_pvalues)
+  if (anyNA(input_pvalues)) {
+    warning("`input_pvalues` contains NAs, which will be removed.")
+    input_pvalues <- input_pvalues[complete.cases(input_pvalues), ]
   }
-  input_pvalues <- matrix(as.numeric(input_pvalues), nrow = nrow(input_pvalues))
-  if (sum(stats::complete.cases(input_pvalues)) < nrow(input_pvalues)) {
-    warning("input_pvalues contains NAs to be removed from analysis")
+  if (nrow(input_pvalues) == 0) {
+    stop("`input_pvalues` does not contain valid rows.")
   }
-  input_pvalues <- input_pvalues[stats::complete.cases(input_pvalues), ]
-  if (!is.null(nrow(input_pvalues)) && nrow(input_pvalues) < 1) {
-    stop("input_pvalues doesn't have valid p-values")
-  }
-
+  
+  ## Precompute threshold values
   pcut <- seq(0.1, 0.8, 0.1)
-  frac1 <- rep(0, 8)
-  frac2 <- rep(0, 8)
-  frac12 <- rep(0, 8)
-  for (i in 1:8) {
-    frac1[i] <- mean(input_pvalues[, 1] >= pcut[i]) / (1 - pcut[i])
-    frac2[i] <- mean(input_pvalues[, 2] >= pcut[i]) / (1 - pcut[i])
-    frac12[i] <- mean(input_pvalues[, 2] >= pcut[i] & input_pvalues[, 1] >= pcut[i]) / (1 - pcut[i])^2
-  }
-
-  ## use the median estimates for pi00 ##
-
+  one_minus_pcut <- 1 - pcut
+  one_minus_pcut_sq <- one_minus_pcut^2
+  
+  ## Calculate fractions using vectorized operations
+  frac1 <- colMeans(outer(input_pvalues[, 1], pcut, `>=`)) / one_minus_pcut
+  frac2 <- colMeans(outer(input_pvalues[, 2], pcut, `>=`)) / one_minus_pcut
+  frac12 <- colMeans(outer(input_pvalues[, 2], pcut, `>=`) & outer(input_pvalues[, 1], pcut, `>=`)) / one_minus_pcut_sq
+  
+  ## Estimate alpha00
   alpha00 <- min(frac12[pcut == lambda], 1)
-
-  ## alpha1 is the proportion of nulls for first p-value
-  ## alpha2 is the proportion of nulls for second p-value
-
-  if (stats::ks.test(input_pvalues[, 1], "punif", 0, 1, alternative = "greater")$p > 0.05) alpha1 <- 1 else alpha1 <- min(frac1[pcut == lambda], 1)
-  if (stats::ks.test(input_pvalues[, 2], "punif", 0, 1, alternative = "greater")$p > 0.05) alpha2 <- 1 else alpha2 <- min(frac2[pcut == lambda], 1)
-
-
+  
+  ## Estimate alpha1 and alpha2
+  alpha1 <- if (stats::ks.test(input_pvalues[, 1], "punif", 0, 1, alternative = "greater")$p > 0.05) 1 else min(frac1[pcut == lambda], 1)
+  alpha2 <- if (stats::ks.test(input_pvalues[, 2], "punif", 0, 1, alternative = "greater")$p > 0.05) 1 else min(frac2[pcut == lambda], 1)
+  
+  ## Estimate other proportions
   if (alpha00 == 1) {
-    alpha01 <- 0
-    alpha10 <- 0
-    alpha11 <- 0
+    alpha01 <- alpha10 <- alpha11 <- 0
   } else {
-    if (alpha1 == 1 & alpha2 == 1) {
-      alpha01 <- 0
-      alpha10 <- 0
-      alpha11 <- 0
+    alpha01 <- alpha1 - alpha00
+    alpha10 <- alpha2 - alpha00
+    alpha01 <- max(0, alpha01)
+    alpha10 <- max(0, alpha10)
+    
+    if (alpha1 == 1 && alpha2 == 1) {
       alpha00 <- 1
-    }
-
-    if (alpha1 == 1 & alpha2 != 1) {
-      alpha10 <- 0
+      alpha01 <- alpha10 <- alpha11 <- 0
+    } else if ((1 - alpha00 - alpha01 - alpha10) < 0) {
       alpha11 <- 0
-      alpha01 <- alpha1 - alpha00
-      alpha01 <- max(0, alpha01)
-      alpha00 <- 1 - alpha01
-    }
-
-    if (alpha1 != 1 & alpha2 == 1) {
-      alpha01 <- 0
-      alpha11 <- 0
-      alpha10 <- alpha2 - alpha00
-      alpha10 <- max(0, alpha10)
-      alpha00 <- 1 - alpha10
-    }
-
-    if (alpha1 != 1 & alpha2 != 1) {
-      alpha10 <- alpha2 - alpha00
-      alpha10 <- max(0, alpha10)
-      alpha01 <- alpha1 - alpha00
-      alpha01 <- max(0, alpha01)
-
-      if ((1 - alpha00 - alpha01 - alpha10) < 0) {
-        alpha11 <- 0
-        alpha10 <- 1 - alpha1
-        alpha01 <- 1 - alpha2
-        alpha00 <- 1 - alpha10 - alpha01
-      } else {
-        alpha11 <- 1 - alpha00 - alpha01 - alpha10
-      }
+      alpha10 <- 1 - alpha1
+      alpha01 <- 1 - alpha2
+      alpha00 <- 1 - alpha10 - alpha01
+    } else {
+      alpha11 <- 1 - alpha00 - alpha01 - alpha10
     }
   }
-  alpha.null <- list(alpha10 = alpha10, alpha01 = alpha01, alpha00 = alpha00, alpha1 = alpha1, alpha2 = alpha2)
-  return(alpha.null)
+  
+  ## Return results
+  list(alpha10 = alpha10, alpha01 = alpha01, alpha00 = alpha00, alpha1 = alpha1, alpha2 = alpha2)
 }
 
 
-
 # Internal function: DLASSO_fun
-# A function perform de-biased lasso estimator used by function "microHIMA"
+# A function perform de-biased lasso estimator used by function "hima_microbiome"
 
 DLASSO_fun <- function(X, Y) {
+  set.seed(1029)
+  
   n <- dim(X)[1]
   p <- dim(X)[2]
   fit <- glmnet(X, Y, alpha = 1)
@@ -340,15 +302,13 @@ DLASSO_fun <- function(X, Y) {
   beta_1_hat <- beta_0[1] + sum(R * E) / sum(R * X[, 1]) #  The de-biased lasso estimator
   ##
   sigma_e2 <- sum(E^2) / (n - length(which(beta_0 != 0)))
-
+  
   sigma_beta1_hat <- sqrt(sigma_e2) * sqrt(sum(R^2)) / abs(sum(R * X[, 1]))
-
+  
   results <- c(beta_1_hat, sigma_beta1_hat)
   return(results)
 }
 
-
-
 # Internal function: rdirichlet
 # A function generate random number from Dirichlet distribution.
 
@@ -360,104 +320,92 @@ rdirichlet <- function(n = 1, alpha) {
 
 
 
-# Internal function: DACT
+# Internal function: DACT (optimized)
 # A function to perform Divide-Aggregate Composite-null Test (DACT) by Liu et al. (2020).
 # p value is corrected by JC method Jin and Cai (2007).
-# This function is used in eHIMA
+# This function is used in hima_efficient
 
 DACT <- function(p_a, p_b) {
-  Z_a <- stats::qnorm(p_a, lower.tail = F)
-  Z_b <- stats::qnorm(p_b, lower.tail = F)
+  Z_a <- stats::qnorm(p_a, lower.tail = FALSE)
+  Z_b <- stats::qnorm(p_b, lower.tail = FALSE)
   pi0a <- 1 - .nonnullPropEst(Z_a, 0, 1)
   pi0b <- 1 - .nonnullPropEst(Z_b, 0, 1)
-  if (pi0a > 1) {
-    pi0a <- 1
-  }
-  if (pi0b > 1) {
-    pi0b <- 1
-  }
-  p.mat <- cbind(p_a, p_b)
-  p3 <- (apply(p.mat, 1, max))^2
+  pi0a <- min(pi0a, 1)
+  pi0b <- min(pi0b, 1)
+  
+  p3 <- (pmax(p_a, p_b))^2
   wg1 <- pi0a * (1 - pi0b)
   wg2 <- (1 - pi0a) * pi0b
   wg3 <- pi0a * pi0b
   wg.sum <- wg1 + wg2 + wg3
   wg.std <- c(wg1, wg2, wg3) / wg.sum
+  
   p_dact <- wg.std[1] * p_a + wg.std[2] * p_b + wg.std[3] * p3
   p_dact <- .JCCorrect(p_dact)
   return(p_dact)
 }
 
 .JCCorrect <- function(pval) {
-  z <- stats::qnorm(pval, lower.tail = F)
+  z <- stats::qnorm(pval, lower.tail = FALSE)
   res <- .nullParaEst(z)
-  pval.JC <- stats::pnorm(z, mean = res$mu, sd = res$s, lower.tail = F)
-  return(pval.JC)
+  stats::pnorm(z, mean = res$mu, sd = res$s, lower.tail = FALSE)
 }
 
 .nonnullPropEst <- function(x, u, sigma) {
   z <- (x - u) / sigma
-  xi <- c(0:100) / 100
+  xi <- seq(0, 1, length.out = 101)
   tmax <- sqrt(log(length(x)))
   tt <- seq(0, tmax, 0.1)
-
-  epsest <- NULL
-
+  
+  weights <- 1 - abs(xi)  # Weights based on xi
+  epsest <- numeric(length(tt))  # Preallocate results
+  
   for (j in seq_along(tt)) {
     t <- tt[j]
-    f <- t * xi
-    f <- exp(f^2 / 2)
-    w <- (1 - abs(xi))
-    co <- 0 * xi
-
-    for (i in 1:101) {
-      co[i] <- mean(cos(t * xi[i] * z))
-    }
-    epshat <- 1 - sum(w * f * co) / sum(w)
-    epsest <- c(epsest, epshat)
+    f <- exp((t * xi)^2 / 2)
+    co <- rowMeans(cos(outer(t * xi, z, `*`)))  # Calculate cosine terms
+    epsest[j] <- 1 - sum(weights * f * co) / sum(weights)
   }
-  return(epsest = max(epsest))
+  
+  max(epsest)
 }
 
 .nullParaEst <- function(x, gamma = 0.1) {
   n <- length(x)
-  t <- c(1:1000) / 200
-
+  t <- seq(0.005, 5, length.out = 1000)  # Define range for smoother spacing
+  
   gan <- n^(-gamma)
-  that <- 0
-  shat <- 0
-  uhat <- 0
-  epshat <- 0
-
-  phiplus <- rep(1, 1000)
-  phiminus <- rep(1, 1000)
-  dphiplus <- rep(1, 1000)
-  dphiminus <- rep(1, 1000)
-  phi <- rep(1, 1000)
-  dphi <- rep(1, 1000)
-
-  for (i in 1:1000) {
-    s <- t[i]
-    phiplus[i] <- mean(cos(s * x))
-    phiminus[i] <- mean(sin(s * x))
-    dphiplus[i] <- -mean(x * sin(s * x))
-    dphiminus[i] <- mean(x * cos(s * x))
-    phi[i] <- sqrt(phiplus[i]^2 + phiminus[i]^2)
+  
+  # Compute cos(s * x) and sin(s * x) using outer for vectorization
+  cos_vals <- outer(t, x, FUN = function(t, x) cos(t * x))  # Matrix of size (length(t), length(x))
+  sin_vals <- outer(t, x, FUN = function(t, x) sin(t * x))  # Matrix of size (length(t), length(x))
+  
+  # Compute phi and its derivatives
+  phiplus <- rowMeans(cos_vals)  # Mean of each row
+  phiminus <- rowMeans(sin_vals)  # Mean of each row
+  dphiplus <- -rowMeans(sweep(sin_vals, 2, x, `*`))  # Broadcasting x across columns
+  dphiminus <- rowMeans(sweep(cos_vals, 2, x, `*`))  # Broadcasting x across columns
+  phi <- sqrt(phiplus^2 + phiminus^2)  # Magnitude of phiplus and phiminus
+  
+  # Find the first index where phi - gan <= 0
+  ind <- which((phi - gan) <= 0)[1]
+  
+  if (is.na(ind)) {
+    stop("Unable to find a suitable index where phi - gan <= 0.")
   }
-
-  ind <- min(c(1:1000)[(phi - gan) <= 0])
+  
+  # Extract values at the identified index
   tt <- t[ind]
   a <- phiplus[ind]
   b <- phiminus[ind]
   da <- dphiplus[ind]
   db <- dphiminus[ind]
   c <- phi[ind]
-
-  that <- tt
-  shat <- -(a * da + b * db) / (tt * c * c)
-  shat <- sqrt(shat)
-  uhat <- -(da * b - db * a) / (c * c)
+  
+  # Compute final estimates
+  shat <- sqrt(-(a * da + b * db) / (tt * c^2))
+  uhat <- -(da * b - db * a) / (c^2)
   epshat <- 1 - c * exp((tt * shat)^2 / 2)
-
-  return(musigma = list(mu = uhat, s = shat))
+  
+  list(mu = uhat, s = shat)
 }
